// import 'source-map-support/register';
import {
  ApprovalInfo,
  CardEvent,
  CardEventSelectedItems,
  ChangeContact,
  EventKeyBody,
  PhotoEventBody,
  ScanCodeEventBody,
  WecomEventBody,
  WecomEventFunction,
} from './def';
import { Adapter } from 'koishi';
import HttpServer from './http';
import { BotConfig, WeComBot } from './bot';
import { AdapterConfig } from './utils';

declare module 'koishi' {
  interface Session {
    wecom?: WecomEventBody;
  }
  interface EventMap {
    'wecom/enter-agent': WecomEventFunction;
    'wecom/subscribe': WecomEventFunction;
    'wecom/unsubscribe': WecomEventFunction;
    'wecom/location': WecomEventFunction<{
      Latitude: number;
      Longitude: number;
      Precision: number;
    }>;
    'wecom/batch-job-result': WecomEventFunction<{
      BatchJob: {
        JobId: string;
        JobType: string;
        ErrCode: number;
        ErrMsg: string;
      };
    }>;
    'wecom/click': WecomEventFunction<EventKeyBody>;
    'wecom/view': WecomEventFunction<EventKeyBody>;
    'wecom/scancode-push': WecomEventFunction<ScanCodeEventBody>;
    'wecom/scancode-waitmsg': WecomEventFunction<ScanCodeEventBody>;
    'wecom/pic-sysphoto': WecomEventFunction<PhotoEventBody>;
    'wecom/pic-photo-or-album': WecomEventFunction<PhotoEventBody>;
    'wecom/pic-weixin': WecomEventFunction<PhotoEventBody>;
    'wecom/location-select': WecomEventFunction<
      EventKeyBody & {
        SendLocationInfo: {
          Location_X: number;
          Location_Y: number;
          Scale: number;
          Label: string;
          Poiname: string;
        };
      }
    >;
    'wecom/share-agent-change': WecomEventFunction;
    'wecom/share-chain-change': WecomEventFunction;
    'wecom/template-card-event': WecomEventFunction<
      CardEvent & {
        SelectedItems: CardEventSelectedItems;
      }
    >;
    'wecom/template-card-menu-event': WecomEventFunction<CardEvent>;
    'wecom/open-approval-change': WecomEventFunction<{
      ApprovalInfo: ApprovalInfo;
    }>;
    'wecom/change-contact': WecomEventFunction<ChangeContact>;
  }
}

export * from './def';
export * from './bot';
export * from './http';

export const adapterPlugin = Adapter.define<BotConfig, AdapterConfig>(
  'wecom',
  WeComBot,
  HttpServer,
);
export default adapterPlugin;
